//Copyright 2005 Tijmen van gulik
//Distributed on an as is basis. Apache License, Version 2.0
//http://www.apache.org/licenses/LICENSE-2.0

unit Utilities;

interface

uses Classes,Windows;

function GetWinTempDir : string;
function GenerateTempFileName(sPath:string) : string;
function ShellExecute_Simple( FileName, Parameters, sDirectory : string;bFillResult : boolean) : TStrings;
function ShellExecute_Wait(hWnd: HWND; Operation, FileName, Parameters,
                            Directory: PAnsiChar; ShowCmd: Integer;bFillResult : boolean) : TStrings;
procedure SaveToWideStringFile(sFile : string;sl : TStrings);
procedure LoadFromWideStringFile(sFile : string;sl : TStrings);
procedure ShellExecuteDirect_Simple( FileName, Parameters, sDirectory : string;bFillResult : boolean);
procedure ShellExecuteDirect(hWnd: HWND; Operation, FileName, Parameters,
                            Directory: PAnsiChar; ShowCmd: Integer);
function findLowerCaseListValue(sl : TStrings;sValue : string;bInsertWhenNotFound : Boolean) : integer;

implementation

uses SysUtils,ShellAPI,Forms,GpTextStream;

// Get the windows temp dir
function GetWinTempDir : string;
var sPtrTempPath:PChar;
begin
  sPtrTempPath:=StrAlloc(1024);
  GetTempPath(1000,sPtrTempPath);
  result:=sPtrTempPath;
  StrDispose(sPtrTempPath);
end;

//Returns a Temporary File in the given path.
function GenerateTempFileName(sPath:string) : string;
var sPtrTempName:PChar;
begin
  sPtrTempName:=StrAlloc(1024);
  GetTempFileName(PChar(sPath),'TMP',0,sPtrTempName);
  result:=sPtrTempName;
  StrDispose(sPtrTempName);
end;

function ShellExecute_Simple( FileName, Parameters, sDirectory : string;bFillResult : boolean) : TStrings;
begin
  Result:=ShellExecute_Wait(0, 'OPEN', PChar(FileName), PChar(Parameters),
                            PChar(sDirectory),0,bFillResult);
end;

function ShellExecute_Wait(hWnd: HWND; Operation, FileName, Parameters,
                            Directory: PAnsiChar; ShowCmd: Integer;bFillResult : boolean) : TStrings;
var Info : TShellExecuteInfo;
    sOutPutFile : string;
begin
  if bFillResult then
  begin
    sOutPutFile:=GenerateTempFileName(GetWinTempDir);
    Parameters:=PAnsiChar('/C "'+FileName+'" '+Parameters+' >"'+sOutPutFile+'"');
    FileName:=PAnsiChar('cmd');
  end;
  Result:=nil;
  Info.cbSize:=Sizeof(TShellExecuteInfo);
  Info.lpVerb:=Operation;
  Info.fMask:= SEE_MASK_FLAG_DDEWAIT or SEE_MASK_NOCLOSEPROCESS;
  Info.hInstApp:=HINSTANCE;
  Info.hInstApp:=0;
  Info.lpFile:=FileName;
  Info.lpParameters:= Parameters;
  Info.lpDirectory:=Directory;
  Info.Wnd:=hWnd;
  Info.nShow:=ShowCmd;
  if not ShellExecuteEx(@Info) then
    raise Exception.Create(SysErrorMessage(GetLastError));
//  WaitForSingleObject( Info.hProcess,INFINITE);
  //The following line does works but does not sove the problem of the
  // changed cursor and stoped animatin
  while WaitForSingleObject( Info.hProcess, 100 )=WAIT_TIMEOUT do
    Forms.Application.ProcessMessages;
  if bFillResult then
  begin
    Result:=TStringList.Create;
    if FileExists(sOutPutFile) then
    begin
      Result.LoadFromFile(sOutPutFile);
      SysUtils.DeleteFile(sOutPutFile);
    end;
  end;
end;

procedure ShellExecuteDirect_Simple( FileName, Parameters, sDirectory : string;bFillResult : boolean) ;
begin
  ShellExecuteDirect(0, 'OPEN', PChar(FileName), PChar(Parameters),
                            PChar(sDirectory),1);
end;

procedure ShellExecuteDirect(hWnd: HWND; Operation, FileName, Parameters,
                            Directory: PAnsiChar; ShowCmd: Integer);
var Info : TShellExecuteInfo;
begin
  Info.cbSize:=Sizeof(TShellExecuteInfo);
  Info.lpVerb:=Operation;
  Info.fMask:= SEE_MASK_FLAG_DDEWAIT or SEE_MASK_NOCLOSEPROCESS;
  Info.hInstApp:=HINSTANCE;
  Info.hInstApp:=0;
  Info.lpFile:=FileName;
  Info.lpParameters:= Parameters;
  Info.lpDirectory:=Directory;
  Info.Wnd:=hWnd;
  Info.nShow:=ShowCmd;
  if not ShellExecuteEx(@Info) then
    raise Exception.Create(SysErrorMessage(GetLastError));
end;

procedure LoadFromWideStringFile(sFile : string;sl : TStrings);
var oStream : TGpTextStream;
   oFileStream : TFileStream;
begin
  oFileStream := TFileStream.Create(sFile,fmOpenRead);
  oStream :=TGpTextStream.Create(oFileStream,tsaccRead,[],CP_UNICODE);
  try
    sl.Clear;
    while oStream.Position<>oStream.Size do
     sl.Add(oStream.Readln);
  finally
    oStream.Free;
    oFileStream.Free;
  end;
end;

procedure SaveToWideStringFile(sFile : string;sl : TStrings);
var oStream : TGpTextStream;
   oFileStream : TFileStream;
   i : integer;
begin
  oFileStream := TFileStream.Create(sFile,fmCreate);
  oStream :=TGpTextStream.Create(oFileStream,tsaccWrite,[tscfUnicode,tscfWriteUTF8BOM],CP_UNICODE);
  try
    for i:=0 to sl.Count-1 do
      oStream.Writeln(WideString(sl[i]));
  finally
    oStream.Free;
    oFileStream.Free;
  end;
end;

function findLowerCaseListValue(sl : TStrings;sValue : string;bInsertWhenNotFound : Boolean) : integer;
var i : integer;
begin
  Result:=-1;
  for i:=0 to sl.Count-1 do
    if trim(Lowercase(sl[i]))=sValue then
    begin
      Result:=i;
      break;
    end;
  if Result=-1 then
  begin
    sl.Append(sValue);
    Result:=sl.Count-1;
  end;
end;
end.
